#include <EEPROM.h>
#include <LiquidCrystal.h>
#include <MIDI.h>
MIDI_CREATE_DEFAULT_INSTANCE();

// Store names in flash
const char s0[] PROGMEM = "UTIL";
const char s1[] PROGMEM = "CLEAN";
const char s2[] PROGMEM = "SLO";
const char s3[] PROGMEM = "PLEXI";
const char s4[] PROGMEM = "TUBE";
const char s5[] PROGMEM = "SLUDGE";
const char s6[] PROGMEM = "DELAY";
const char s7[] PROGMEM = "FX";
const char s8[] PROGMEM = "MISC";
const char s9[] PROGMEM = "MAIN";       //10
const char s10[] PROGMEM = "BANK  11";
const char s11[] PROGMEM = "BANK  12";
const char s12[] PROGMEM = "BANK  13";
const char s13[] PROGMEM = "BANK  14";
const char s14[] PROGMEM = "BANK  15";
const char s15[] PROGMEM = "BANK  16";
const char s16[] PROGMEM = "BANK  17";
const char s17[] PROGMEM = "BANK  18";
const char s18[] PROGMEM = "BANK  19";
const char s19[] PROGMEM = "BANK  20";  //20
const char s20[] PROGMEM = "BANK  21";
const char s21[] PROGMEM = "BANK  22";
const char s22[] PROGMEM = "BANK  23";
const char s23[] PROGMEM = "SYNTH";
const char s24[] PROGMEM = "BASS";      //25
const char s25[] PROGMEM = "prog";      //PC name 1
const char s26[] PROGMEM = "chug";
const char s27[] PROGMEM = "solo";
const char s28[] PROGMEM = "clean";
const char s29[] PROGMEM = "acoustic";
const char s30[] PROGMEM = "fuzz";
const char s31[] PROGMEM = "wah";
const char s32[] PROGMEM = "whammy";
const char s33[] PROGMEM = "chorus";
const char s34[] PROGMEM = "reverb";     //10
const char s35[] PROGMEM = "name 11";
const char s36[] PROGMEM = "name 12";
const char s37[] PROGMEM = "name 13";
const char s38[] PROGMEM = "name 14";
const char s39[] PROGMEM = "name 15";
const char s40[] PROGMEM = " ";          //16


const char *const string_table[] PROGMEM = {s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12, s13, s14, s15, s16, s17, s18, s19, s20, s21, s22, s23, s24, s25, s26, s27, s28, s29, s30, s31, s32, s33, s34, s35, s36, s37, s38, s39, s40};
char buffer[9];

// initialize LCD
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);

// 4021
int latchPin = A2;
int dataPin = A0;
int clockPin = A1;

// variables
unsigned long sinceLastPress = 0;
unsigned long sinceCompared = 0;
int address = 0;
// D10 LOW == RPi ready
byte piCheck = 1;

byte switchRead = 255;
byte LastSwitchRead = 0;
char button = ' ';

byte pc = 125;    //V-amps use 0-124 & 127 for tuner
byte pcName = 40;
byte pcRP = 0;
bool pcGate = 0;
bool pcMuff = 0;
byte pcTube = 0;
byte pcTap = 0;

byte bank = 0;
byte currentBank = 0;

bool editMode = false;
bool piReady = false;
bool wasRead = false;
bool tapped = false;    //Tap function switch

void setup() {

  currentBank = 10;     //read last preset from EEPROM ?
  bank = currentBank;

  pinMode(A3, INPUT);   //Gate in
  pinMode(A5, INPUT);   //Muff in
  pinMode(A4, OUTPUT);  //Gate out
  pinMode(7, OUTPUT);  //Muff out  //A6 NFG!
  pinMode(A7, OUTPUT);  //tube LSB
  pinMode(6, OUTPUT);   //tube MSB

  pinMode(10, INPUT);   //pi check
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, INPUT);
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  // initialize MIDI
  //MIDI.begin(1);


  digitalWrite(A4, LOW);  //set bump pins?
  digitalWrite(A6, LOW);
}

void loop() {
  for (int i = 1; i <= 500000; i = i * 2) {
    digitalWrite(13, HIGH);
    digitalWrite(A4, HIGH);
    delayMicroseconds(i);
    digitalWrite(A4, LOW);
    delayMicroseconds(10);
    digitalWrite(13, LOW);

    digitalWrite(13, HIGH);
    digitalWrite(7, HIGH);
    delayMicroseconds(i);
    digitalWrite(7, LOW);
    piReady = digitalRead(A5);
    lcd.print(i);
    lcd.setCursor(0, 1);
    lcd.print(piReady);
    delay(1000);
    digitalWrite(13, HIGH);
    lcd.clear();
  }
}

//functions

void updateLCD() {
  int bankIndex = bank - 1;             //read bank name from flash
  strcpy_P(buffer, (char *)pgm_read_word(&(string_table[bankIndex])));
  lcd.clear();
  if (bank < 10) {
    lcd.setCursor(1, 0);                //lcd.setCursor(col, row)
  } else {
    lcd.setCursor(0, 0);
  }
  lcd.print(bank);
  lcd.setCursor(3, 0);
  lcd.print(buffer);
  if (piReady == true) {
    lcd.setCursor(11, 0);
    if (pcRP < 10) {
      lcd.print(" RP0");
    } else {
      lcd.print(" RP");
    }
    lcd.print(pcRP);
  } else {
    lcd.setCursor(11, 0);
    lcd.print(" ... ");
  }

  strcpy_P(buffer, (char *)pgm_read_word(&(string_table[pcName])));
  lcd.setCursor(1, 1);
  lcd.print(button);
  lcd.setCursor(3, 1);
  lcd.print(buffer);
  lcd.setCursor(12, 1);
  switch (pcTap) {
    case 0:
      lcd.print(F("TAP "));
      break;
    case 1:
      lcd.print(F("VERB"));
      break;
    case 2:
      lcd.print(F("MUTE"));
      break;
    case 3:
      lcd.print(F("M V3"));
      break;
    case 4:
      lcd.print(F("M V2"));
      break;
    case 5:
      lcd.print(F("FX  "));
      break;
    case 6:
      lcd.print(F("RPbp"));
      break;
    case 7:
      lcd.print(F("DET "));
      break;
    default:
      lcd.print(F("TAP "));
      break;
  }
}

void editLCD() {
  strcpy_P(buffer, (char *)pgm_read_word(&(string_table[pcName]))); //read PC name from flash
  lcd.clear();
  if (bank < 10) {
    lcd.setCursor(1, 0);                //lcd.setCursor(col, row)
  } else {
    lcd.setCursor(0, 0);
  }
  lcd.print(bank);
  lcd.setCursor(3, 0);
  lcd.print(buffer);

  lcd.setCursor(1, 1);
  lcd.print(button);
  lcd.setCursor(3, 1);
  lcd.print("RP");
  lcd.print(pcRP);
  lcd.print(" T");
  lcd.print(pcTube);
  lcd.print(" *");
  switch (pcTap) {
    case 0:
      lcd.print(F("TAP "));
      break;
    case 1:
      lcd.print(F("VERB"));
      break;
    case 2:
      lcd.print(F("MUTE"));
      break;
    case 3:
      lcd.print(F("M V3"));
      break;
    case 4:
      lcd.print(F("M V2"));
      break;
    case 5:
      lcd.print(F("FX  "));
      break;
    case 6:
      lcd.print(F("RPon"));
      break;
    case 7:
      lcd.print(F("RPmo"));
      break;
    default:
      lcd.print(F("DET "));
      break;
  }

}

void readMem() {
  byte byte2 = EEPROM.read(pc + 128);
  byte byte3 = EEPROM.read(pc + 256);
  pcRP = EEPROM.read(pc);
  pcTap = byte2 & 7;
  pcName = byte2 >> 4;
  pcName = pcName & 15;
  pcName = pcName + 25;
  pcGate = bitRead(byte3, 3);
  pcMuff = bitRead(byte3, 2);
  pcTube = byte3 & 3;
  tapped = false;
}
void writeMem() {
  pcGate = digitalRead(A3);
  pcMuff = digitalRead(A5);
  EEPROM.update(pc, pcRP);
  byte byte2 = pcName - 25;
  byte2 = byte2 << 4;
  byte2 = byte2 | pcTap;
  EEPROM.update(pc + 128, byte2);
  byte byte3 = pcTube;
  bitWrite(byte3, 3, pcGate);
  bitWrite(byte3, 2, pcMuff);
  EEPROM.update(pc + 256, byte3);
}

void bump() {
  bool currentGate = digitalRead(A3);
  bool currentMuff = digitalRead(A5);
  while (currentGate != pcGate) {
    digitalWrite(A4, HIGH);
    delay(1);
    digitalWrite(A4, LOW);
    delay(1);
    currentGate = digitalRead(A3);
  }
  while (currentMuff != pcMuff) {
    digitalWrite(A6, HIGH);
    delay(1);
    digitalWrite(A6, LOW);
    delay(1);
    currentMuff = digitalRead(A5);
  }
  digitalWrite(A7, pcTube & 1); //LSB
  digitalWrite(6, pcTube & 2);  //MSB
}

byte shiftIn(int myDataPin, int myClockPin) {
  int i;
  int temp = 0;
  int pinState;
  byte myDataIn = 0;

  pinMode(myClockPin, OUTPUT);
  pinMode(myDataPin, INPUT);


  for (i = 7; i >= 0; i--)
  {
    digitalWrite(myClockPin, 0);
    delayMicroseconds(2);
    temp = digitalRead(myDataPin);
    if (temp) {
      pinState = 1;
      myDataIn = myDataIn | (1 << i);
    }
    else {
      pinState = 0;
    }
    digitalWrite(myClockPin, 1);
  }
  return myDataIn;
}

void tapFunc() {
  tapped = !tapped;
  switch (pcTap) {
    case 0:
      // "TAP "
      MIDI.sendControlChange(64, 100, 2);
      MIDI.sendControlChange(64, 100, 3);
      break;
    case 1:
      // "VERB"
      if (tapped) {
        MIDI.sendControlChange(22, 127, 2);
        MIDI.sendControlChange(22, 127, 3);
      } else {
        MIDI.sendControlChange(22, 0, 2);
        MIDI.sendControlChange(22, 0, 3);
      }
      break;
    case 2:
      // "MUTE"
      if (tapped) {
        MIDI.sendProgramChange(127, 2);    //V-amp 127 = tuner
        MIDI.sendProgramChange(127, 3);
      } else {
        MIDI.sendProgramChange(pc, 2);
        MIDI.sendProgramChange(pc, 3);
      }
      break;
    case 3:
      // "M V3"
      if (tapped) {
        MIDI.sendProgramChange(127, 3);
      } else {
        MIDI.sendProgramChange(pc, 3);
      }
      break;
    case 4:
      // "M V2"
      if (tapped) {
        MIDI.sendProgramChange(127, 2);
      } else {
        MIDI.sendProgramChange(pc, 2);
      }
      break;
    case 5:
      // "FX  "
      if (tapped) {
        MIDI.sendControlChange(21, 127, 2);
        MIDI.sendControlChange(21, 127, 3);
      } else {
        MIDI.sendControlChange(21, 0, 2);
        MIDI.sendControlChange(21, 0, 3);
      }
      break;
    case 6:
      // "RPon"
      if (!tapped) {                      //default true
        MIDI.sendProgramChange(10, 1);    //RP10 = bypass
      } else {
        MIDI.sendProgramChange(pcRP, 1);
      }
      break;
    case 7:
      // "RPmo"
      break;
    default:
      // "DET "
      MIDI.sendControlChange(80, 80, 2);
      break;
  }
}
